@@ -93,11 +93,13 @@ from qtpy import QtGui as QG
 from qtpy import QtWidgets as QW
 
 from plotpy.config import CONF
+from plotpy.constants import SHAPE_Z_OFFSET
 from plotpy.coords import axes_to_canvas, canvas_to_axes
-from plotpy.items.shape.marker import Marker
+from plotpy.items import AnnotatedPolygon, Marker, PolygonShape
 
 if TYPE_CHECKING:
     from qtpy.QtCore import QPoint
+    from qtpy.QtGui import QMouseEvent
 
     from plotpy.plot.base import BasePlot
 
@@ -259,7 +261,7 @@ class MouseEventMatch(EventMatch):
         """Return the set of event types handled by this event match"""
         return frozenset((self.evt_type,))
 
-    def __call__(self, event: QG.QMouseEvent) -> bool:
+    def __call__(self, event: QMouseEvent) -> bool:
         """Returns True if the event matches the event match
 
         Args:
@@ -282,7 +284,7 @@ class MouseEventMatch(EventMatch):
 
 
 class MouseMoveMatch(MouseEventMatch):
-    def __call__(self, event: QG.QMouseEvent) -> bool:
+    def __call__(self, event: QMouseEvent) -> bool:
         """Returns True if the event matches the event match
 
         Args:
@@ -655,7 +657,7 @@ class DragHandler(QC.QObject):
         self.last = QC.QPointF(pos)
         return dx, dy
 
-    def start_tracking(self, _filter: StatefulEventFilter, event: QC.QEvent) -> None:
+    def start_tracking(self, _filter: StatefulEventFilter, event: QMouseEvent) -> None:
         """Start tracking the mouse movement
 
         Args:
@@ -664,7 +666,7 @@ class DragHandler(QC.QObject):
         """
         self.start = self.last = QC.QPointF(event.pos())
 
-    def start_moving(self, filter: StatefulEventFilter, event: QC.QEvent) -> None:
+    def start_moving(self, filter: StatefulEventFilter, event: QMouseEvent) -> None:
         """Start moving the mouse
 
         Args:
@@ -673,7 +675,7 @@ class DragHandler(QC.QObject):
         """
         return self.move(filter, event)
 
-    def stop_tracking(self, _filter: StatefulEventFilter, _event: QC.QEvent) -> None:
+    def stop_tracking(self, _filter: StatefulEventFilter, _event: QMouseEvent) -> None:
         """Stop tracking the mouse movement
 
         Args:
@@ -683,7 +685,7 @@ class DragHandler(QC.QObject):
         pass
         # filter.plot.canvas().setMouseTracking(self.parent_tracking)
 
-    def stop_notmoving(self, filter: StatefulEventFilter, event: QC.QEvent) -> None:
+    def stop_notmoving(self, filter: StatefulEventFilter, event: QMouseEvent) -> None:
         """Stop tracking the mouse movement when the mouse is not moving.
 
         Args:
@@ -692,7 +694,7 @@ class DragHandler(QC.QObject):
         """
         self.stop_tracking(filter, event)
 
-    def stop_moving(self, filter: StatefulEventFilter, event: QC.QEvent) -> None:
+    def stop_moving(self, filter: StatefulEventFilter, event: QMouseEvent) -> None:
         """Stop moving the mouse
 
         Args:
@@ -701,7 +703,7 @@ class DragHandler(QC.QObject):
         """
         self.stop_tracking(filter, event)
 
-    def move(self, filter: StatefulEventFilter, event: QC.QEvent) -> None:
+    def move(self, filter: StatefulEventFilter, event: QMouseEvent) -> None:
         """Move the mouse
 
         Args:
@@ -755,28 +757,28 @@ class ClickHandler(QC.QObject):
 class PanHandler(DragHandler):
     cursor = QC.Qt.ClosedHandCursor
 
-    def move(self, filter: StatefulEventFilter, event: QC.QEvent) -> None:
+    def move(self, filter: StatefulEventFilter, event: QMouseEvent) -> None:
         """Move the mouse.
 
         Args:
             filter: The StatefulEventFilter instance.
             event: The mouse event.
         """
-        x_state, y_state = self.get_move_state(filter, event.pos())
+        x_state, y_state = self.get_move_state(filter, QC.QPointF(event.pos()))
         filter.plot.do_pan_view(x_state, y_state)
 
 
 class ZoomHandler(DragHandler):
     cursor = QC.Qt.SizeAllCursor
 
-    def move(self, filter: StatefulEventFilter, event: QC.QEvent) -> None:
+    def move(self, filter: StatefulEventFilter, event: QMouseEvent) -> None:
         """Move the mouse.
 
         Args:
             filter: The StatefulEventFilter instance.
             event: The mouse event.
         """
-        x_state, y_state = self.get_move_state(filter, event.pos())
+        x_state, y_state = self.get_move_state(filter, QC.QPointF(event.pos()))
         filter.plot.do_zoom_view(x_state, y_state)
 
 
@@ -1133,18 +1135,18 @@ class QtDragHandler(DragHandler):
     """Class to handle drag events using Qt signals."""
 
     #: Signal emitted by QtDragHandler when starting tracking
-    SIG_START_TRACKING = QC.Signal(object, "QEvent")
+    SIG_START_TRACKING = QC.Signal(object, "QMouseEvent")
 
     #: Signal emitted by QtDragHandler when stopping tracking and not moving
-    SIG_STOP_NOT_MOVING = QC.Signal(object, "QEvent")
+    SIG_STOP_NOT_MOVING = QC.Signal(object, "QMouseEvent")
 
     #: Signal emitted by QtDragHandler when stopping tracking and moving
-    SIG_STOP_MOVING = QC.Signal(object, "QEvent")
+    SIG_STOP_MOVING = QC.Signal(object, "QMouseEvent")
 
     #: Signal emitted by QtDragHandler when moving
-    SIG_MOVE = QC.Signal(object, "QEvent")
+    SIG_MOVE = QC.Signal(object, "QMouseEvent")
 
-    def start_tracking(self, filter: StatefulEventFilter, event: QC.QEvent) -> None:
+    def start_tracking(self, filter: StatefulEventFilter, event: QMouseEvent) -> None:
         """Starts tracking the drag event.
 
         Args:
@@ -1154,7 +1156,7 @@ class QtDragHandler(DragHandler):
         DragHandler.start_tracking(self, filter, event)
         self.SIG_START_TRACKING.emit(filter, event)
 
-    def stop_notmoving(self, filter: StatefulEventFilter, event: QC.QEvent) -> None:
+    def stop_notmoving(self, filter: StatefulEventFilter, event: QMouseEvent) -> None:
         """Stops tracking when the drag event is not moving.
 
         Args:
@@ -1163,9 +1165,8 @@ class QtDragHandler(DragHandler):
         """
         self.SIG_STOP_NOT_MOVING.emit(filter, event)
 
-    def stop_moving(self, filter: StatefulEventFilter, event: QC.QEvent) -> None:
-        """
-        Stops the movement of the drag event.
+    def stop_moving(self, filter: StatefulEventFilter, event: QMouseEvent) -> None:
+        """Stops the movement of the drag event.
 
         Args:
             filter: The StatefulEventFilter instance.
@@ -1173,7 +1174,7 @@ class QtDragHandler(DragHandler):
         """
         self.SIG_STOP_MOVING.emit(filter, event)
 
-    def move(self, filter: StatefulEventFilter, event: QC.QEvent) -> None:
+    def move(self, filter: StatefulEventFilter, event: QMouseEvent) -> None:
         """Handles the move event.
 
         Args:
@@ -1631,6 +1632,10 @@ class RectangularSelectionHandler(DragHandler):
     ) -> None:
         super().__init__(filter, btn, mods, start_state)
         self.avoid_null_shape = False
+        self.setup_shape_cb = None
+        self.shape = None
+        self.shape_h0 = None
+        self.shape_h1 = None
 
     def set_shape(
         self,
@@ -1655,7 +1660,7 @@ class RectangularSelectionHandler(DragHandler):
         self.setup_shape_cb = setup_shape_cb
         self.avoid_null_shape = avoid_null_shape
 
-    def start_tracking(self, filter: StatefulEventFilter, event: QC.QEvent) -> None:
+    def start_tracking(self, filter: StatefulEventFilter, event: QMouseEvent) -> None:
         """Start tracking the mouse movement for selection and interaction.
 
         Args:
@@ -1664,7 +1669,7 @@ class RectangularSelectionHandler(DragHandler):
         """
         self.start = self.last = QC.QPointF(event.pos())
 
-    def start_moving(self, filter: StatefulEventFilter, event: QC.QEvent) -> None:
+    def start_moving(self, filter: StatefulEventFilter, event: QMouseEvent) -> None:
         """Start moving the object.
 
         Args:
@@ -1676,7 +1681,7 @@ class RectangularSelectionHandler(DragHandler):
         if self.avoid_null_shape:
             self.start -= QC.QPointF(1, 1)
         self.shape.move_local_point_to(self.shape_h0, self.start)
-        self.shape.move_local_point_to(self.shape_h1, event.pos())
+        self.shape.move_local_point_to(self.shape_h1, QC.QPointF(event.pos()))
         self.start_moving_action(filter, event)
         if self.setup_shape_cb is not None:
             self.setup_shape_cb(self.shape)
@@ -1684,7 +1689,7 @@ class RectangularSelectionHandler(DragHandler):
         filter.plot.replot()
 
     def start_moving_action(
-        self, filter: StatefulEventFilter, event: QC.QEvent
+        self, filter: StatefulEventFilter, event: QMouseEvent
     ) -> None:
         """Start moving the object.
 
@@ -1694,18 +1699,18 @@ class RectangularSelectionHandler(DragHandler):
         """
         pass
 
-    def move(self, filter: StatefulEventFilter, event: QC.QEvent) -> None:
+    def move(self, filter: StatefulEventFilter, event: QMouseEvent) -> None:
         """Move the object.
 
         Args:
             filter: The StatefulEventFilter instance.
             event: The mouse event.
         """
-        self.shape.move_local_point_to(self.shape_h1, event.pos())
+        self.shape.move_local_point_to(self.shape_h1, QC.QPointF(event.pos()))
         self.move_action(filter, event)
         filter.plot.replot()
 
-    def move_action(self, filter: StatefulEventFilter, event: QC.QEvent) -> None:
+    def move_action(self, filter: StatefulEventFilter, event: QMouseEvent) -> None:
         """Move the object.
 
         Args:
@@ -1714,7 +1719,7 @@ class RectangularSelectionHandler(DragHandler):
         """
         pass
 
-    def stop_moving(self, filter: StatefulEventFilter, event: QC.QEvent) -> None:
+    def stop_moving(self, filter: StatefulEventFilter, event: QMouseEvent) -> None:
         """Stop moving the object.
 
         Args:
@@ -1725,20 +1730,22 @@ class RectangularSelectionHandler(DragHandler):
         self.stop_moving_action(filter, event)
         filter.plot.replot()
 
-    def stop_moving_action(self, filter: StatefulEventFilter, event: QC.QEvent) -> None:
+    def stop_moving_action(
+        self, filter: StatefulEventFilter, event: QMouseEvent
+    ) -> None:
         """Stop moving the object.
 
         Args:
             filter: The StatefulEventFilter instance.
             event: The mouse event.
         """
-        self.SIG_END_RECT.emit(filter, self.start, event.pos())
+        self.SIG_END_RECT.emit(filter, self.start, QC.QPointF(event.pos()))
 
 
 class PointSelectionHandler(RectangularSelectionHandler):
     """Class to handle point selections."""
 
-    def stop_notmoving(self, filter: StatefulEventFilter, event: QC.QEvent) -> None:
+    def stop_notmoving(self, filter: StatefulEventFilter, event: QMouseEvent) -> None:
         """Stop the point selection when the point is not moving.
 
         Args:
@@ -1751,14 +1758,181 @@ class PointSelectionHandler(RectangularSelectionHandler):
 class ZoomRectHandler(RectangularSelectionHandler):
     """Class to handle zoom rectangles."""
 
-    def stop_moving_action(self, filter: StatefulEventFilter, event: QC.QEvent) -> None:
+    def stop_moving_action(
+        self, filter: StatefulEventFilter, event: QMouseEvent
+    ) -> None:
         """Stop moving the object.
 
         Args:
             filter: The StatefulEventFilter instance.
             event: The mouse event.
         """
-        filter.plot.do_zoom_rect_view(self.start, event.pos())
+        filter.plot.do_zoom_rect_view(self.start, QC.QPointF(event.pos()))
+
+
+class MultilineSelectionHandler(DragHandler):
+    """
+    A handler for handling multi-line selections.
+
+    This handler extends the DragHandler to handle multi-line selections.
+
+    Args:
+        filter: The StatefulEventFilter instance.
+        btn: The mouse button to match.
+        mods: The keyboard modifiers to match. (default: QC.Qt.NoModifier)
+        start_state: The starting state. (default: 0)
+        closed: Whether the polygon should be closed. (default: False)
+    """
+
+    SIG_END_POLYLINE = QC.Signal(object, np.ndarray)
+
+    def __init__(
+        self,
+        filter: StatefulEventFilter,
+        btn: int,
+        mods: QC.Qt.KeyboardModifiers = QC.Qt.NoModifier,
+        start_state: int = 0,
+        closed: bool = False,
+    ) -> None:
+        super().__init__(filter, btn, mods, start_state)
+        self.closed = closed
+        filter.add_event(
+            start_state,
+            KeyEventMatch((QC.Qt.Key_Enter, QC.Qt.Key_Return, QC.Qt.Key_Space)),
+            self.accept_polygonshape,
+            start_state,
+        )
+        filter.add_event(
+            start_state,
+            KeyEventMatch((QC.Qt.Key_Backspace, QC.Qt.Key_Escape)),
+            self.cancel_point,
+            start_state,
+        )
+        self.init_pos: QC.QPointF | None = None
+        self.shape: PolygonShape | AnnotatedPolygon | None = None
+        self.current_handle: int | None = None
+        self.setup_shape_cb: Callable | None = None
+
+    def set_shape(
+        self,
+        shape: PolygonShape | AnnotatedPolygon,
+        setup_shape_cb: Callable | None = None,
+    ) -> None:
+        """Set the shape.
+
+        Args:
+            shape: The shape.
+            setup_shape_cb: The setup shape callback.
+        """
+        self.shape = shape
+        self.setup_shape_cb = setup_shape_cb
+
+    def start_tracking(self, filter: StatefulEventFilter, event: QMouseEvent) -> None:
+        """Starts tracking the drag event.
+
+        Args:
+            filter: The StatefulEventFilter instance.
+            event: The QC.QEvent instance.
+        """
+        if self.init_pos is None:
+            self.init_pos = QC.QPointF(event.pos())
+
+    def start_moving(self, filter: StatefulEventFilter, event: QMouseEvent) -> None:
+        """Start moving the object.
+
+        Args:
+            filter: The StatefulEventFilter instance.
+            event: The mouse event.
+        """
+        if self.init_pos is None:
+            return
+        if self.shape.plot() is None:
+            self.shape.set_points(None)
+            filter.plot.add_item_with_z_offset(self.shape, SHAPE_Z_OFFSET)
+            self.shape.setZ(filter.plot.get_max_z() + 1)
+            if self.setup_shape_cb is not None:
+                self.setup_shape_cb(self.shape)
+            self.shape.add_local_point(self.init_pos)
+        self.current_handle = self.shape.add_local_point(QC.QPointF(event.pos()))
+        self.shape.set_closed(self.closed and len(self.shape.get_points()) > 2)
+        self.shape.show()
+        filter.plot.replot()
+
+    def move(self, filter: StatefulEventFilter, event: QMouseEvent) -> None:
+        """
+        Handle mouse move event to update the position of the last point.
+
+        Args:
+            filter: The plot filter.
+            event: The mouse event.
+        """
+        self.shape.move_local_point_to(self.current_handle, QC.QPointF(event.pos()))
+        filter.plot.replot()
+
+    def stop_notmoving(self, filter: StatefulEventFilter, event: QMouseEvent) -> None:
+        """Stops tracking when the drag event is not moving.
+
+        Args:
+            filter: The StatefulEventFilter instance.
+            event: The QC.QEvent instance.
+        """
+        points = self.shape.get_points()
+        if len(points) > 1:
+            self.accept_polygonshape(filter, event)
+
+    def stop_moving(self, filter: StatefulEventFilter, event: QMouseEvent) -> None:
+        """Stops the movement of the drag event.
+
+        Args:
+            filter: The StatefulEventFilter instance.
+            event: The QC.QEvent instance
+        """
+        pos = QC.QPointF(event.pos())
+        if self.init_pos == pos:
+            self.shape.del_point(-1)
+        else:
+            self.shape.move_local_point_to(self.current_handle, pos)
+        filter.plot.replot()
+
+    def accept_polygonshape(
+        self, filter: StatefulEventFilter, event: QMouseEvent
+    ) -> None:
+        """Accept the polygon shape.
+
+        Args:
+            filter: The StatefulEventFilter instance.
+            event: The mouse event.
+        """
+        if self.shape.plot() is None:
+            return
+        filter.plot.del_item(self.shape)
+        self.init_pos = None
+        self.current_handle = None
+        self.SIG_END_POLYLINE.emit(filter, self.shape.get_points())
+
+    def cancel_point(self, filter: StatefulEventFilter, event: QMouseEvent) -> None:
+        """
+        Cancel the last point or remove the shape if it has less than 3 points.
+
+        Args:
+            filter: The plot filter.
+            event: The triggering event.
+        """
+        points = self.shape.get_points()
+        if points is None or self.shape.plot() is None:
+            return
+        if len(points) <= 2:
+            filter.plot.del_item(self.shape)
+            self.init_pos = None
+            self.shape.closed = False
+        else:
+            if self.current_handle:
+                newh = self.shape.del_point(self.current_handle)
+            else:
+                newh = self.shape.del_point(-1)
+            self.current_handle = newh
+            self.shape.set_closed(self.closed and len(self.shape.get_points()) > 2)
+        filter.plot.replot()
 
 
 def setup_standard_tool_filter(filter: StatefulEventFilter, start_state: int) -> int:
