@@ -284,55 +284,55 @@ def _imread_pil(filename, to_grayscale=False, **kwargs):
         "I;16",
         "I;16",
     )
-    img = PIL.Image.open(filename)
-    base, ext = osp.splitext(filename)
-    ext = ext.lower()
-    if ext in [".tif", ".tiff"]:
-        # try to know if multiple pages
-        nb_pages = 0
-        while True:
-            try:
-                img.seek(nb_pages)
-                nb_pages += 1
-            except EOFError:
-                break
-        if nb_pages > 1:
-            for i in range(nb_pages):
-                img.seek(i)
-                filename = base
-                filename += "_{i:d}".format(i=i)
-                filename += ext
-                img.save(filename)
-            if nb_pages == 2:
-                # possibility to be TIFF file with thumbnail and full image
-                # --> try to load full image (second one)
-                filename = base + "_{i:d}".format(i=1) + ext
-            else:
-                # we don't know which one must be loaded --> load first image
-                filename = base + "_{i:d}".format(i=0) + ext
+    with PIL.Image.open(filename) as img:
+        base, ext = osp.splitext(filename)
+        ext = ext.lower()
+        if ext in [".tif", ".tiff"]:
+            # try to know if multiple pages
+            nb_pages = 0
+            while True:
+                try:
+                    img.seek(nb_pages)
+                    nb_pages += 1
+                except EOFError:
+                    break
+            if nb_pages > 1:
+                for i in range(nb_pages):
+                    img.seek(i)
+                    filename = base
+                    filename += "_{i:d}".format(i=i)
+                    filename += ext
+                    img.save(filename)
+                if nb_pages == 2:
+                    # possibility to be TIFF file with thumbnail and full image
+                    # --> try to load full image (second one)
+                    filename = base + "_{i:d}".format(i=1) + ext
+                else:
+                    # we don't know which one must be loaded --> load first image
+                    filename = base + "_{i:d}".format(i=0) + ext
 
-    img = PIL.Image.open(filename)
-    if img.mode in ("CMYK", "YCbCr"):
-        # Converting to RGB
-        img = img.convert("RGB")
-    if to_grayscale and img.mode in ("RGB", "RGBA", "RGBX"):
-        # Converting to grayscale
-        img = img.convert("L")
-    elif "A" in img.mode or (img.mode == "P" and "transparency" in img.info):
-        img = img.convert("RGBA")
-    elif img.mode == "P":
-        img = img.convert("RGB")
-    try:
-        dtype, extra = DTYPES[img.mode]
-    except KeyError:
-        raise RuntimeError(f"{img.mode} mode is not supported")
-    shape = (img.size[1], img.size[0])
-    if extra is not None:
-        shape += (extra,)
-    try:
-        return np.array(img, dtype=np.dtype(dtype)).reshape(shape)
-    except SystemError:
-        return np.array(img.getdata(), dtype=np.dtype(dtype)).reshape(shape)
+    with PIL.Image.open(filename) as img:
+        if img.mode in ("CMYK", "YCbCr"):
+            # Converting to RGB
+            img = img.convert("RGB")
+        if to_grayscale and img.mode in ("RGB", "RGBA", "RGBX"):
+            # Converting to grayscale
+            img = img.convert("L")
+        elif "A" in img.mode or (img.mode == "P" and "transparency" in img.info):
+            img = img.convert("RGBA")
+        elif img.mode == "P":
+            img = img.convert("RGB")
+        try:
+            dtype, extra = DTYPES[img.mode]
+        except KeyError:
+            raise RuntimeError(f"{img.mode} mode is not supported")
+        shape = (img.size[1], img.size[0])
+        if extra is not None:
+            shape += (extra,)
+        try:
+            return np.array(img, dtype=np.dtype(dtype)).reshape(shape)
+        except SystemError:
+            return np.array(img.getdata(), dtype=np.dtype(dtype)).reshape(shape)
 
 
 def _imwrite_pil(filename, arr):
@@ -374,7 +374,7 @@ def _import_dcm():
     # is to check if pydicom is installed:
     # pylint: disable=import-outside-toplevel
     # pylint: disable=import-error
-    from pydicom import dicomio  # type:ignore # noqa: F401
+    from pydicom import dcmread  # type:ignore # noqa: F401
 
     logger.setLevel(logging.WARNING)
 
@@ -383,9 +383,9 @@ def _imread_dcm(filename, **kwargs):
     """Open DICOM image with pydicom and return a NumPy array"""
     # pylint: disable=import-outside-toplevel
     # pylint: disable=import-error
-    from pydicom import dicomio  # type:ignore
+    from pydicom import dcmread  # type:ignore
 
-    dcm = dicomio.read_file(filename, force=True)
+    dcm = dcmread(filename, force=True)
     # **********************************************************************
     # The following is necessary until pydicom numpy support is improved:
     # (after that, a simple: 'arr = dcm.PixelArray' will work the same)
@@ -620,6 +620,7 @@ register_serializable_items(
         "AnnotatedObliqueRectangle",
         "AnnotatedEllipse",
         "AnnotatedCircle",
+        "AnnotatedPolygon",
         "LabelItem",
         "LegendBoxItem",
         "SelectedLegendBoxItem",
